@@ -649,7 +649,7 @@ void AIGateway::showBlockingDialog(const std::string & text, const std::vector<C
 				auto ratio = static_cast<float>(danger) / hero->getTotalStrength();
 
 				answer = topObj->id == goalObjectID; // no if we do not aim to visit this object
-				logAi->trace("Query hook: %s(%s) by %s danger ratio %f", target.toString(), topObj->getObjectName(), hero.name, ratio);
+				logAi->trace("Query hook: %s(%s) by %s danger ratio %f", target.toString(), topObj->getObjectName(), hero.name(), ratio);
 
 				if(cb->getObj(goalObjectID, false))
 				{
@@ -1321,7 +1321,7 @@ bool AIGateway::moveHeroToTile(int3 dst, HeroPtr h)
 		{
 			destinationTeleport = exitId;
 			if(exitPos.valid())
-				destinationTeleportPos = h->convertFromVisitablePos(exitPos);
+				destinationTeleportPos = exitPos;
 			cb->moveHero(*h, h->pos, false);
 			destinationTeleport = ObjectInstanceID();
 			destinationTeleportPos = int3(-1);
@@ -1331,17 +1331,32 @@ bool AIGateway::moveHeroToTile(int3 dst, HeroPtr h)
 		auto doChannelProbing = [&]() -> void
 		{
 			auto currentPos = h->visitablePos();
-			auto currentExit = getObj(currentPos, true)->id;
+			auto currentTeleport = getObj(currentPos, true);
 
-			status.setChannelProbing(true);
-			for(auto exit : teleportChannelProbingList)
-				doTeleportMovement(exit, int3(-1));
-			teleportChannelProbingList.clear();
-			status.setChannelProbing(false);
+			if(currentTeleport)
+			{
+				auto currentExit = currentTeleport->id;
 
-			doTeleportMovement(currentExit, currentPos);
+				status.setChannelProbing(true);
+				for(auto exit : teleportChannelProbingList)
+					doTeleportMovement(exit, int3(-1));
+				teleportChannelProbingList.clear();
+				status.setChannelProbing(false);
+
+				doTeleportMovement(currentExit, currentPos);
+			}
+			else
+			{
+				logAi->debug("Unexpected channel probbing at " + currentPos.toString());
+
+				teleportChannelProbingList.clear();
+				status.setChannelProbing(false);
+			}
 		};
 
+		teleportChannelProbingList.clear();
+		status.setChannelProbing(false);
+
 		for(; i > 0; i--)
 		{
 			int3 currentCoord = path.nodes[i].coord;
@@ -1559,7 +1574,7 @@ void AIGateway::requestActionASAP(std::function<void()> whatToDo)
 
 void AIGateway::lostHero(HeroPtr h)
 {
-	logAi->debug("I lost my hero %s. It's best to forget and move on.", h.name);
+	logAi->debug("I lost my hero %s. It's best to forget and move on.", h.name());
 }
 
 void AIGateway::answerQuery(QueryID queryID, int selection)
