@@ -10,9 +10,8 @@
 #pragma once
 
 #include "../../../lib/VCMI_Lib.h"
-#include "../../../lib/CBuildingHandler.h"
-#include "../../../lib/CCreatureHandler.h"
-#include "../../../lib/CTownHandler.h"
+#include "../../../lib/mapObjects/CGTownInstance.h"
+#include "../../../lib/mapObjects/CGHeroInstance.h"
 #include "../AIUtility.h"
 
 namespace NKAI
@@ -21,6 +20,7 @@ namespace NKAI
 struct HeroPtr;
 class AIGateway;
 class FuzzyHelper;
+class Nullkiller;
 
 namespace Goals
 {
@@ -73,7 +73,10 @@ namespace Goals
 		CAPTURE_OBJECT,
 		SAVE_RESOURCES,
 		STAY_AT_TOWN_BEHAVIOR,
-		STAY_AT_TOWN
+		STAY_AT_TOWN,
+		EXPLORATION_BEHAVIOR,
+		EXPLORATION_POINT,
+		EXPLORE_NEIGHBOUR_TILE
 	};
 
 	class DLL_EXPORT TSubgoal : public std::shared_ptr<AbstractGoal>
@@ -105,7 +108,7 @@ namespace Goals
 		int objid; SETTER(int, objid)
 		int aid; SETTER(int, aid)
 		int3 tile; SETTER(int3, tile)
-		HeroPtr hero; SETTER(HeroPtr, hero)
+		const CGHeroInstance * hero; SETTER(CGHeroInstance *, hero)
 		const CGTownInstance *town; SETTER(CGTownInstance *, town)
 		int bid; SETTER(int, bid)
 
@@ -118,6 +121,7 @@ namespace Goals
 			objid = -1;
 			tile = int3(-1, -1, -1);
 			town = nullptr;
+			hero = nullptr;
 			bid = -1;
 			goldCost = 0;
 		}
@@ -128,7 +132,7 @@ namespace Goals
 			return const_cast<AbstractGoal *>(this);
 		}
 
-		virtual TGoalVec decompose() const
+		virtual TGoalVec decompose(const Nullkiller * ai) const
 		{
 			return TGoalVec();
 		}
@@ -146,6 +150,11 @@ namespace Goals
 		virtual bool hasHash() const { return false; }
 
 		virtual uint64_t getHash() const { return 0; }
+
+		virtual ITask * asTask()
+		{
+			throw std::runtime_error("Abstract goal is not a task");
+		}
 		
 		bool operator!=(const AbstractGoal & g) const
 		{
@@ -164,9 +173,11 @@ namespace Goals
 		//TODO: make accept work for std::shared_ptr... somehow
 		virtual void accept(AIGateway * ai) = 0; //unhandled goal will report standard error
 		virtual std::string toString() const = 0;
-		virtual HeroPtr getHero() const = 0;
+		virtual const CGHeroInstance * getHero() const = 0;
 		virtual ~ITask() {}
 		virtual int getHeroExchangeCount() const = 0;
+		virtual bool isObjectAffected(ObjectInstanceID h) const = 0;
+		virtual std::vector<ObjectInstanceID> getAffectedObjects() const = 0;
 	};
 }
 
@@ -180,11 +191,7 @@ public:
 	{
 	}
 
-	virtual ~cannotFulfillGoalException() throw ()
-	{
-	};
-
-	const char * what() const throw () override
+	const char * what() const noexcept override
 	{
 		return msg.c_str();
 	}
@@ -203,11 +210,7 @@ public:
 		msg = goal->toString();
 	}
 
-	virtual ~goalFulfilledException() throw ()
-	{
-	};
-
-	const char * what() const throw () override
+	const char * what() const noexcept override
 	{
 		return msg.c_str();
 	}
