@@ -353,7 +353,10 @@ void MapRendererFow::renderTile(IMapRendererContext & context, Canvas & target,
 		size_t pseudorandomNumber = ((coordinates.x * 997) ^ (coordinates.y * 1009)) / 101;
 		size_t imageIndex = pseudorandomNumber % fogOfWarFullHide->size();
 
-		target.draw(fogOfWarFullHide->getImage(imageIndex), Point(0, 0));
+		if (context.showSpellRange(coordinates))
+			target.drawColor(Rect(0,0,32,32), Colors::BLACK);
+		else
+			target.draw(fogOfWarFullHide->getImage(imageIndex), Point(0, 0));
 	}
 	else
 	{
@@ -363,6 +366,9 @@ void MapRendererFow::renderTile(IMapRendererContext & context, Canvas & target,
 
 uint8_t MapRendererFow::checksum(IMapRendererContext & context, const int3 & coordinates)
 {
+	if (context.showSpellRange(coordinates))
+		return 0xff - 2;
+
 	const NeighborTilesInfo neighborInfo(context, coordinates);
 	int retBitmapID = neighborInfo.getBitmapID();
 	if(retBitmapID < 0)
@@ -658,15 +664,15 @@ size_t MapRendererPath::selectImageArrow(bool reachableToday, const int3 & curr,
 	// is (directionToArrowIndex[7][5])
 	//
 	const static size_t directionToArrowIndex[9][9] = {
-		{16, 17, 18, 7,  0, 19, 6,  5,  0 },
-		{8,  9,  18, 7,  0, 19, 6,  0,  20},
-		{8,  1,  10, 7,  0, 19, 0,  21, 20},
-		{24, 17, 18, 15, 0, 0,  6,  5,  4 },
+		{16, 17, 18, 7,  0, 19, 6,  5,  12},
+		{8,  9,  18, 7,  0, 19, 6,  13, 20},
+		{8,  1,  10, 7,  0, 19, 14, 21, 20},
+		{24, 17, 18, 15, 0, 11, 6,  5,  4 },
 		{0,  0,  0,  0,  0, 0,  0,  0,  0 },
-		{8,  1,  2,  0,  0, 11, 22, 21, 20},
-		{24, 17, 0,  23, 0, 3,  14, 5,  4 },
-		{24, 0,  2,  23, 0, 3,  22, 13, 4 },
-		{0,  1,  2,  23, 0, 3,  22, 21, 12}
+		{8,  1,  2,  15, 0, 11, 22, 21, 20},
+		{24, 17, 10, 23, 0, 3,  14, 5,  4 },
+		{24, 9,  2,  23, 0, 3,  22, 13, 4 },
+		{16, 1,  2,  23, 0, 3,  22, 21, 12}
 	};
 
 	size_t enterDirection = (curr.x - next.x + 1) + 3 * (curr.y - next.y + 1);
