@@ -10,8 +10,10 @@
 #pragma once
 
 #include "CArtPlace.h"
+#include "Scrollable.h"
 
 class CButton;
+class BackpackScroller;
 
 class CArtifactsOfHeroBase : virtual public CIntObject
 {
@@ -50,6 +52,7 @@ protected:
 	std::vector<ArtPlacePtr> backpack;
 	std::shared_ptr<CButton> leftBackpackRoll;
 	std::shared_ptr<CButton> rightBackpackRoll;
+	std::shared_ptr<BackpackScroller> backpackScroller;
 
 	const std::vector<Point> slotPos =
 	{
@@ -67,3 +70,13 @@ protected:
 	// Assigns an artifacts to an artifact place depending on it's new slot ID
 	virtual void setSlotData(ArtPlacePtr artPlace, const ArtifactPosition & slot);
 };
+
+class BackpackScroller : public Scrollable
+{
+	CArtifactsOfHeroBase * owner;
+
+	void scrollBy(int distance) override;
+
+public:
+	BackpackScroller(CArtifactsOfHeroBase * owner, const Rect & dimensions);
+};
