@@ -35,15 +35,11 @@ class FirstLaunchView : public QWidget
 	void languageSelected(const QString & languageCode);
 
 	// Tab Heroes III Data
-	bool heroesDataUpdate();
 	bool heroesDataDetect();
 
 	void heroesDataMissing();
 	void heroesDataDetected();
 
-	void heroesLanguageUpdate();
-	void forceHeroesLanguage(const QString & language);
-
 	QString getHeroesInstallDir();
 	void extractGogData();
 	void copyHeroesData(const QString & path = {}, bool move = false);
@@ -64,6 +60,9 @@ class FirstLaunchView : public QWidget
 public:
 	explicit FirstLaunchView(QWidget * parent = nullptr);
 
+	// Tab Heroes III Data
+	bool heroesDataUpdate();
+
 public slots:
 
 private slots:
@@ -82,19 +81,14 @@ private slots:
 
 	void on_pushButtonGogInstall_clicked();
 
-	void on_comboBoxLanguage_currentIndexChanged(int index);
-
 	void on_pushButtonPresetBack_clicked();
 
 	void on_pushButtonPresetNext_clicked();
 
 	void on_pushButtonDiscord_clicked();
 
-	void on_pushButtonSlack_clicked();
-
 	void on_pushButtonGithub_clicked();
 
 private:
 	Ui::FirstLaunchView * ui;
-
 };
