@@ -70,9 +70,8 @@ void CModListView::dropEvent(QDropEvent* event)
 	if(mimeData->hasUrls())
 	{
 		const QList<QUrl> urlList = mimeData->urls();
-
 		for (const auto & url : urlList)
-			manualInstallFile(url);
+			manualInstallFile(url.toLocalFile());
 	}
 }
 
@@ -102,9 +101,8 @@ void CModListView::setupModsView()
 	}
 	else //default //TODO: default high-DPI scaling
 	{
-		ui->allModsView->setColumnWidth(ModFields::NAME, 185);
+		ui->allModsView->setColumnWidth(ModFields::NAME, 220);
 		ui->allModsView->setColumnWidth(ModFields::TYPE, 75);
-		ui->allModsView->setColumnWidth(ModFields::VERSION, 60);
 	}
 
 	ui->allModsView->resizeColumnToContents(ModFields::STATUS_ENABLED);
@@ -130,6 +128,12 @@ CModListView::CModListView(QWidget * parent)
 
 	setAcceptDrops(true);
 
+	ui->uninstallButton->setIcon(QIcon{":/icons/mod-delete.png"});
+	ui->enableButton->setIcon(QIcon{":/icons/mod-enabled.png"});
+	ui->disableButton->setIcon(QIcon{":/icons/mod-disabled.png"});
+	ui->updateButton->setIcon(QIcon{":/icons/mod-update.png"});
+	ui->installButton->setIcon(QIcon{":/icons/mod-download.png"});
+
 	setupModModel();
 	setupFilterModel();
 	setupModsView();
@@ -145,13 +149,14 @@ CModListView::CModListView(QWidget * parent)
 	{
 		manager->resetRepositories();
 	}
-	
-#ifdef Q_OS_IOS
+
+#ifdef VCMI_MOBILE
 	for(auto * scrollWidget : {
 		(QAbstractItemView*)ui->allModsView,
 		(QAbstractItemView*)ui->screenshotsList})
 	{
-		QScroller::grabGesture(scrollWidget, QScroller::LeftMouseButtonGesture);
+		Helper::enableScrollBySwiping(scrollWidget);
+
 		scrollWidget->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 		scrollWidget->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 	}
@@ -324,9 +329,6 @@ QString CModListView::genModInfoText(CModEntry & mod)
 
 	for(const auto & language : Languages::getLanguageList())
 	{
-		if (!language.hasTranslation)
-			continue;
-
 		QString languageID = QString::fromStdString(language.identifier);
 
 		if (languageID != baseLanguageID && !mod.getValue(languageID).isValid())
@@ -393,14 +395,18 @@ void CModListView::selectMod(const QModelIndex & index)
 	}
 	else
 	{
-		auto mod = modModel->getMod(index.data(ModRoles::ModNameRole).toString());
+		const auto modName = index.data(ModRoles::ModNameRole).toString();
+		auto mod = modModel->getMod(modName);
 
 		ui->modInfoBrowser->setHtml(genModInfoText(mod));
 		ui->changelogBrowser->setHtml(genChangelogText(mod));
 
-		bool hasInvalidDeps = !findInvalidDependencies(index.data(ModRoles::ModNameRole).toString()).empty();
-		bool hasBlockingMods = !findBlockingMods(index.data(ModRoles::ModNameRole).toString()).empty();
-		bool hasDependentMods = !findDependentMods(index.data(ModRoles::ModNameRole).toString(), true).empty();
+		Helper::enableScrollBySwiping(ui->modInfoBrowser);
+		Helper::enableScrollBySwiping(ui->changelogBrowser);
+
+		bool hasInvalidDeps = !findInvalidDependencies(modName).empty();
+		bool hasBlockingMods = !findBlockingMods(modName).empty();
+		bool hasDependentMods = !findDependentMods(modName, true).empty();
 
 		ui->disableButton->setVisible(mod.isEnabled());
 		ui->enableButton->setVisible(mod.isDisabled());
@@ -603,7 +609,7 @@ void CModListView::on_installButton_clicked()
 	for(auto & name : modModel->getRequirements(modName))
 	{
 		auto mod = modModel->getMod(name);
-		if(!mod.isInstalled())
+		if(mod.isAvailable())
 			downloadFile(name + ".zip", mod.getValue("download").toString(), name, mbToBytes(mod.getValue("downloadSize").toDouble()));
 		else if(!mod.isEnabled())
 			enableModByName(name);
@@ -624,28 +630,31 @@ void CModListView::on_installButton_clicked()
 
 void CModListView::on_installFromFileButton_clicked()
 {
-	QString filter = tr("All supported files") + " (*.h3m *.vmap *.h3c *.vcmp *.zip *.json);;" + tr("Maps") + " (*.h3m *.vmap);;" + tr("Campaigns") + " (*.h3c *.vcmp);;" + tr("Configs") + " (*.json);;" + tr("Mods") + " (*.zip)";
-	QStringList files = QFileDialog::getOpenFileNames(this, tr("Select files (configs, mods, maps, campaigns) to install..."), QDir::homePath(), filter);
-
-	for (const auto & file : files)
+	// iOS can't display modal dialogs when called directly on button press
+	// https://bugreports.qt.io/browse/QTBUG-98651
+	QTimer::singleShot(0, this, [this]
 	{
-		QUrl url = QUrl::fromLocalFile(file);
-		manualInstallFile(url);
-	}
+		QString filter = tr("All supported files") + " (*.h3m *.vmap *.h3c *.vcmp *.zip *.json);;" + tr("Maps") + " (*.h3m *.vmap);;" + tr("Campaigns") + " (*.h3c *.vcmp);;" + tr("Configs") + " (*.json);;" + tr("Mods") + " (*.zip)";
+		QStringList files = QFileDialog::getOpenFileNames(this, tr("Select files (configs, mods, maps, campaigns) to install..."), QDir::homePath(), filter);
+
+		for(const auto & file : files)
+		{
+			manualInstallFile(file);
+		}
+	});
 }
 
-void CModListView::manualInstallFile(QUrl url)
+void CModListView::manualInstallFile(QString filePath)
 {
-	QString urlStr = url.toString();
-	QString fileName = url.fileName();
-	if(urlStr.endsWith(".zip", Qt::CaseInsensitive))
+	QString fileName = QFileInfo{filePath}.fileName();
+	if(filePath.endsWith(".zip", Qt::CaseInsensitive))
 		downloadFile(fileName.toLower()
 			// mod name currently comes from zip file -> remove suffixes from github zip download
 			.replace(QRegularExpression("-[0-9a-f]{40}"), "")
 			.replace(QRegularExpression("-vcmi-.+\\.zip"), ".zip")
 			.replace("-main.zip", ".zip")
-			, urlStr, "mods", 0);
-	else if(urlStr.endsWith(".json", Qt::CaseInsensitive))
+			, QUrl::fromLocalFile(filePath), "mods");
+	else if(filePath.endsWith(".json", Qt::CaseInsensitive))
 	{
 		QDir configDir(QString::fromStdString(VCMIDirs::get().userConfigPath().string()));
 		QStringList configFile = configDir.entryList({fileName}, QDir::Filter::Files); // case insensitive check
@@ -656,7 +665,7 @@ void CModListView::manualInstallFile(QUrl url)
 			{
 				const auto configFilePath = configDir.filePath(configFile[0]);
 				QFile::remove(configFilePath);
-				QFile::copy(url.toLocalFile(), configFilePath);
+				QFile::copy(filePath, configFilePath);
 
 				// reload settings
 				Helper::loadSettings();
@@ -669,10 +678,15 @@ void CModListView::manualInstallFile(QUrl url)
 		}
 	}
 	else
-		downloadFile(fileName, urlStr, fileName, 0);
+		downloadFile(fileName, QUrl::fromLocalFile(filePath), fileName);
 }
 
 void CModListView::downloadFile(QString file, QString url, QString description, qint64 size)
+{
+	downloadFile(file, QUrl{url}, description, size);
+}
+
+void CModListView::downloadFile(QString file, QUrl url, QString description, qint64 size)
 {
 	if(!dlManager)
 	{
@@ -683,20 +697,17 @@ void CModListView::downloadFile(QString file, QString url, QString description,
 
 		connect(dlManager, SIGNAL(finished(QStringList,QStringList,QStringList)),
 			this, SLOT(downloadFinished(QStringList,QStringList,QStringList)));
-		
+
 		connect(manager.get(), SIGNAL(extractionProgress(qint64,qint64)),
 			this, SLOT(extractionProgress(qint64,qint64)));
-		
+
 		connect(modModel, &CModListModel::dataChanged, filterModel, &QAbstractItemModel::dataChanged);
 
-		
-		QString progressBarFormat = tr("Downloading %s%. %p% (%v MB out of %m MB) finished");
-
-		progressBarFormat.replace("%s%", description);
+		const auto progressBarFormat = tr("Downloading %1. %p% (%v MB out of %m MB) finished").arg(description);
 		ui->progressBar->setFormat(progressBarFormat);
 	}
 
-	dlManager->downloadFile(QUrl(url), file, size);
+	dlManager->downloadFile(url, file, size);
 }
 
 void CModListView::downloadProgress(qint64 current, qint64 max)
@@ -876,6 +887,13 @@ void CModListView::installMods(QStringList archives)
 		auto mod = modModel->getMod(modName);
 		if(mod.isInstalled() && !mod.getValue("keepDisabled").toBool())
 		{
+			for (auto const & dependencyName : mod.getDependencies())
+			{
+				auto dependency = modModel->getMod(dependencyName);
+				if(dependency.isDisabled())
+					manager->enableMod(dependencyName);
+			}
+
 			if(mod.isDisabled() && manager->enableMod(modName))
 			{
 				for(QString child : modModel->getChildren(modName))
