@@ -50,7 +50,7 @@ public:
 	{}
 
 	template <typename Handler>
-	void serialize(Handler &h, const int version)
+	void serialize(Handler &h)
 	{
 		h & r;
 		h & g;
